from media_platform.error_code import ErrorCode
from media_platform.job.result.job_result import JobResult


class JobCallbackFailedResult(JobResult):
    type = None

    def __init__(self, message=None, job_result=None):
        # type: (str, dict) -> None
        super(JobCallbackFailedResult, self).__init__(ErrorCode.job_callback_failed, message)
        self.job_result = job_result

    @classmethod
    def deserialize(cls, data):
        if not data:
            return None

        payload = data.get('payload') or {}
        result = JobResult.deserialize(data)
        result.job_result = payload.get('jobResult')

        result.__class__ = JobCallbackFailedResult

        return result

    def serialize(self):
        data = super(JobCallbackFailedResult, self).serialize()
        data['payload'] = {
            'jobResult': self.job_result
        }

        return data



from media_platform.job.specification import Specification
from media_platform.job.transcode.audio_specification import AudioSpecification
from media_platform.job.transcode.video_specification import VideoSpecification


class StreamType(object):
    audio = 'audio'
    video = 'video'


class StreamSpecification(Specification):
    def __init__(self, stream_type, specification=None, skip=False, copy=False):
        # type: (StreamType, AudioSpecification or VideoSpecification, bool, bool) -> None
        self.stream_type = stream_type
        self.specification = specification
        self.skip = skip
        self.copy = copy

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> StreamSpecification
        stream_type = data['type']

        skip = data.get('skip', False)
        copy = data.get('copy', False)

        specification_data = data.get('specification')
        if skip or copy or not specification_data:
            specification = None
        elif stream_type == StreamType.audio:
            specification = AudioSpecification.deserialize(specification_data)
        elif stream_type == StreamType.video:
            specification = VideoSpecification.deserialize(specification_data)
        else:
            raise ValueError('Specification stream type %s not supported' % stream_type)

        return StreamSpecification(stream_type, specification, skip, copy)

    def serialize(self):
        # type: () -> dict
        return {
            'type': self.stream_type,
            'skip': self.skip,
            'copy': self.copy,
            'specification': self.specification.serialize() if self.specification else None
        }

    def validate(self):
        if self.specification:
            self.specification.validate()



from media_platform.lang.serialization import Serializable, Deserializable


class JobResult(Serializable, Deserializable):
    type = None

    def __init__(self, code=None, message=None):
        # type: (int, str) -> None
        self.code = code or 0
        self.message = message or 'OK'

    @classmethod
    def deserialize(cls, data):
        # type: (dict or None) -> JobResult or None
        if not data:
            return None

        return cls(data['code'], data['message'])

    def serialize(self):
        # type: () -> dict
        return {
            'code': self.code,
            'message': self.message
        }



from media_platform.service.destination import Destination
from media_platform.lang.serialization import Serializable, Deserializable


class ExtractionReportFormat(object):
    csv = 'csv'
    json = 'json'

    @classmethod
    def has_value(cls, value):
        return value in [cls.csv, cls.json]


class ExtractionReport(Serializable, Deserializable):
    def __init__(self, destination, report_format=ExtractionReportFormat.csv):
        # type: (Destination, ExtractionReportFormat) -> None
        super(ExtractionReport, self).__init__()

        self.destination = destination
        self.format = report_format

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExtractionReport
        destination = Destination.deserialize(data['destination'])

        return ExtractionReport(destination, data['format'])

    def serialize(self):
        # type: () -> dict
        return {
            'destination': self.destination.serialize(),
            'format': self.format
        }


from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import FileDescriptor, ACL
from media_platform.service.media_platform_request import MediaPlatformRequest


class UpdateFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(UpdateFileRequest, self).__init__(authenticated_http_client, 'PUT', base_url + '/files', FileDescriptor)

        self.file_id = None
        self.path = None
        self.acl = None

    def set_path(self, path):
        # type: (str) -> UpdateFileRequest
        self.path = path
        return self

    def set_id(self, file_id):
        # type: (str) -> UpdateFileRequest
        self.file_id = file_id
        return self

    def set_acl(self, acl):
        # type: (ACL) -> UpdateFileRequest
        self.acl = acl
        return self

    def execute(self):
        # type: () -> FileDescriptor
        return super(UpdateFileRequest, self).execute()

    def validate(self):
        FileDescriptor.acl_validator(self.acl)

        if self.path is not None:
            FileDescriptor.path_validator(self.path)

        if not self.path and not self.file_id:
            raise ValueError('must provide path or id')

    def _params(self):
        return {
            'id': self.file_id,
            'path': self.path,
            'acl': self.acl,
        }



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.file_service.create_file_request import CreateFileRequest
from media_platform.service.file_service.create_files_response import _CreateFilesResponse
from media_platform.service.media_platform_request import MediaPlatformRequest


class CreateFilesRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(CreateFilesRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/files',
                                                 FileDescriptor)

        self.file_requests = []

    def execute(self):
        # type: () -> Deserializable or None

        #TODO: Replace with single request once server supports that
        file_descriptors = [self.authenticated_http_client.post(self.url, f._params(), self.response_payload_type) for f
                            in self.file_requests]

        return _CreateFilesResponse(file_descriptors)

    def set_file_requests(self, file_requests):
        # type: ([CreateFileRequest]) -> CreateFilesRequest
        self.file_requests = file_requests
        return self

    def add_file(self, file_request):
        # type: (CreateFileRequest) -> CreateFilesRequest
        self.file_requests.append(file_request)
        return self

    def _params(self):
        return {
            'files': [file_request._params() for file_request in self.file_requests]
        }



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.flow_control_service.flow import Flow
from media_platform.service.flow_control_service.flow_state import FlowState
from media_platform.service.flow_control_service.invocation import Invocation
from media_platform.service.media_platform_request import MediaPlatformRequest


class FlowInvocationRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(FlowInvocationRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/flow_control/flow',
                                                    FlowState)

        self.invocation = None  # type: Invocation
        self.flow = None  # type: Flow

    def set_invocation(self, invocation):
        # type: (Invocation) -> FlowInvocationRequest
        self.invocation = invocation
        return self

    def set_flow(self, flow):
        # type: (Flow) -> FlowInvocationRequest
        self.flow = flow
        return self

    def validate(self):
        self.flow.validate()
        self._validate_entry_points()

    def execute(self):
        # type: () -> FlowState
        return super(FlowInvocationRequest, self).execute()

    def _params(self):
        return {
            'invocation': self.invocation.serialize(),
            'flow': self.flow.serialize(),
        }

    def _validate_entry_points(self):
        invalid_entry_points = [e for e in self.invocation.entry_points if e not in self.flow.components]
        if invalid_entry_points:
            raise ValueError('entry points not defined in components: %s' % invalid_entry_points)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.destination import Destination
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class CopyFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(CopyFileRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/copy/file',
                                              FileDescriptor)

        self.source = None
        self.destination = None

    def set_source(self, source):
        # type: (Source) -> CopyFileRequest
        self.source = source
        return self

    def set_destination(self, destination):
        # type: (Destination) -> CopyFileRequest
        self.destination = destination
        return self

    def _params(self):
        return {
            'source': self.source.serialize(),
            'destination': self.destination.serialize(),
        }



import binascii
import os
import time


class Token(object):
    def __init__(self, issuer, subject, verbs=None, issued_at=None, expiration=None,
                 additional_claims=None, token_id=None):
        # type: (str, str, [str], int, int, dict, str) -> None

        self.issuer = issuer
        self.subject = subject

        self.verbs = verbs or list()
        self.issued_at = issued_at or (int(time.time()) - 10)
        self.expiration = expiration or (self.issued_at + 600)
        self.additional_claims = Token._extract_additional_claims(additional_claims) if additional_claims else dict()
        self.id = token_id or binascii.hexlify(os.urandom(6)).decode('utf-8')

    @staticmethod
    def from_claims(data):
        # type: (dict) -> Token
        additional_claims = Token._extract_additional_claims(data)

        return Token(data['iss'], data['sub'], data.get('aud'), data.get('iat'), data.get('exp'),
                     additional_claims, data.get('jti'))

    def to_claims(self):
        # type: () -> dict
        data = {
            'iss': self.issuer,
            'sub': self.subject,
            'aud': self.verbs if self.verbs else list(),
            'iat': self.issued_at,
            'exp': self.expiration,
            'jti': self.id
        }

        data.update(self.additional_claims)

        return data

    @staticmethod
    def _extract_additional_claims(claims):
        # type: (dict) -> dict

        return {k: v for k, v in claims.items() if k not in ['iss', 'sub', 'aud', 'iat', 'exp', 'jti']}



class Serializable(object):

    def serialize(self):
        # type: () -> dict
        raise NotImplementedError()


class Deserializable(object):

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> object
        raise NotImplementedError()



from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.extract_archive.extraction_report import ExtractionReport
from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult


class ExtractArchiveResult(JobResult):
    type = JobType.extract_archive

    def __init__(self, code=None, message=None, extraction_report=None, report_file_descriptor=None):
        # type: (int, str, ExtractionReport, FileDescriptor) -> None
        super(ExtractArchiveResult, self).__init__(code, message)

        self.extraction_report = extraction_report
        self.report_file_descriptor = report_file_descriptor

    @classmethod
    def deserialize(cls, data):
        # type: (dict or None) -> ExtractArchiveResult or None
        if data is None:
            return None

        result = JobResult.deserialize(data)
        payload = data.get('payload') or {}
        extracted_files_report_data = payload.get('extractedFilesReport')
        report_file_descriptor_data = payload.get('reportFileDescriptor')

        result.extraction_report = cls._get_extraction_report(extracted_files_report_data)
        result.report_file_descriptor = cls._get_report_file_descriptor(report_file_descriptor_data)
        result.__class__ = ExtractArchiveResult
        return result

    def serialize(self):
        # type: () -> dict
        data = super(ExtractArchiveResult, self).serialize()
        payload = {}
        if self.extraction_report:
            payload['extractedFilesReport'] = self.extraction_report.serialize()

        if self.report_file_descriptor:
            payload['reportFileDescriptor'] = self.report_file_descriptor.serialize()

        data['payload'] = payload

        return data

    @classmethod
    def _get_extraction_report(cls, extraction_report_data):
        if not extraction_report_data:
            return None

        return ExtractionReport.deserialize(extraction_report_data)

    @classmethod
    def _get_report_file_descriptor(cls, report_file_data):
        if not report_file_data:
            return None

        return FileDescriptor.deserialize(report_file_data)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.create_archive_job import ArchiveSource
from media_platform.service.destination import Destination
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class ZipAlgorithm(object):
    # compresses the objects
    zip = 'zip'
    # copies them as is, for already compressed files such as mp3, mp4, etc.
    store = 'store'

    @classmethod
    def has_value(cls, value):
        return value in [ZipAlgorithm.store, ZipAlgorithm.zip]


class CreateArchiveManifestRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(CreateArchiveManifestRequest, self).__init__(authenticated_http_client, 'POST',
                                                           base_url + '/archive/create/manifest',
                                                           FileDescriptor)
        self.name = None
        self.sources = []
        self.destination = None
        self.algorithm = 'zip'

    def set_sources(self, sources):
        # type: ([Source or ArchiveSource]) -> CreateArchiveManifestRequest
        self.sources = sources
        return self

    def add_sources(self, *sources):
        # type: ([Source or ArchiveSource]) -> CreateArchiveManifestRequest
        self.sources.extend(sources)
        return self

    def set_destination(self, destination):
        # type: (Destination) -> CreateArchiveManifestRequest
        self.destination = destination
        return self

    def set_name(self, name):
        # type: (str) -> CreateArchiveManifestRequest
        self.name = name
        return self

    def set_algorithm(self, algorithm):
        # type: (ZipAlgorithm) -> CreateArchiveManifestRequest
        self.algorithm = algorithm
        return self

    def execute(self):
        # type: () -> FileDescriptor
        return super(CreateArchiveManifestRequest, self).execute()

    def _params(self):
        # type: () -> dict
        return {
            'name': self.name,
            'sources': [source.serialize() for source in self.sources if source],
            'destination': self.destination.serialize(),
            'algorithm': self.algorithm
        }



from media_platform.lang.serialization import Serializable


class Attachment(Serializable):
    def __init__(self, file_name):
        # type: (str) -> None
        super(Attachment, self).__init__()

        self.file_name = file_name

    def set_file_name(self, file_name):
        # type: (str) -> Attachment
        self.file_name = file_name
        return self

    def serialize(self):
        # type: () -> dict
        return {
            'filename': self.file_name
        }



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.job_service.job_list import JobList
from media_platform.service.list_request import _ListRequest


class JobListRequest(_ListRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(JobListRequest, self).__init__(authenticated_http_client, base_url + '/jobs', JobList)

        self.issuer = None
        self.type = None
        self.status = None
        self.group_id = None
        self.file_id = None
        self.path = None

    def set_issuer(self, issuer):
        # type: (str) -> JobListRequest
        self.issuer = issuer
        return self

    def set_type(self, job_type):
        # type: (str) -> JobListRequest
        self.type = job_type
        return self

    def set_status(self, status):
        # type: (str) -> JobListRequest
        self.status = status
        return self

    def set_path(self, path):
        # type: (str) -> JobListRequest
        self.path = path
        return self

    def execute(self):
        # type: () -> JobList
        return super(JobListRequest, self).execute()

    def _params(self):
        # type: () -> dict
        params = super(JobListRequest, self)._params()

        if self.issuer:
            params['issuer'] = self.issuer

        if self.type:
            params['type'] = self.type

        if self.status:
            params['status'] = self.status

        if self.group_id:
            params['groupId'] = self.group_id

        if self.file_id:
            params['fileId'] = self.file_id

        if self.path:
            params['path'] = self.path

        return params



from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult


class SubsetFontResult(JobResult):
    type = JobType.subset_font

    def __init__(self, code=None, message=None, file_descriptor=None):
        # type: (int, str, FileDescriptor) -> None
        super(SubsetFontResult, self).__init__(code, message)
        self.payload = file_descriptor

    @classmethod
    def deserialize(cls, data):
        if data is None:
            return None

        payload_data = data.get('payload')
        result = JobResult.deserialize(data)
        result.payload = FileDescriptor.deserialize(payload_data) if payload_data else None
        result.__class__ = SubsetFontResult
        return result

    def serialize(self):
        data = super(SubsetFontResult, self).serialize()
        data['payload'] = self.payload.serialize() if self.payload else None

        return data



from media_platform.lang.serialization import Deserializable


class Color(Deserializable):
    def __init__(self, r, g, b, pixel_fraction, score):
        # type: (int, int, int, float, float) -> None
        super(Color, self).__init__()
        self.r = r
        self.g = g
        self.b = b
        self.pixel_fraction = pixel_fraction
        self.score = score

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Color
        return Color(data['r'], data['g'], data['b'], data['pixelFraction'], data['score'])



from media_platform.metadata.audio.audio_basic import AudioBasic
from media_platform.metadata.audio.audio_extra import AudioExtra
from media_platform.metadata.file_metadata import FileMetadata, MediaType
from media_platform.service.file_descriptor import FileDescriptor


class AudioFileMetadata(FileMetadata):
    def __init__(self, file_descriptor, basic=None, extra=None):
        # type: (FileDescriptor, AudioBasic, AudioExtra) -> None
        super(AudioFileMetadata, self).__init__(MediaType.audio, file_descriptor, basic)

        self.extra = extra

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> AudioFileMetadata
        if data['mediaType'] != MediaType.audio:
            raise ValueError('not audio metadata')

        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])
        basic = AudioBasic.deserialize(data.get('basic')) if data.get('basic') else None
        extra = AudioExtra.deserialize(data.get('extra')) if data.get('extra') else None

        return AudioFileMetadata(file_descriptor, basic, extra)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_descriptor import ACL, FileDescriptor, FileMimeType
from media_platform.service.file_service.upload_url import UploadUrl
from media_platform.service.media_platform_request import MediaPlatformRequest


class UploadUrlRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(UploadUrlRequest, self).__init__(authenticated_http_client, 'GET', base_url + '/upload/url', UploadUrl)

        self.path = None
        self.mime_type = FileMimeType.defualt
        self.acl = ACL.public

    def set_path(self, path):
        # type: (str) -> UploadUrlRequest
        self.path = path
        return self

    def set_mime_type(self, mime_type):
        # type: (str) -> UploadUrlRequest
        self.mime_type = mime_type
        return self

    def set_acl(self, acl):
        # type: (ACL) -> UploadUrlRequest
        self.acl = acl
        return self

    def execute(self):
        # type: () -> UploadUrl
        return super(UploadUrlRequest, self).execute()

    def validate(self):
        FileDescriptor.path_validator(self.path)
        FileDescriptor.acl_validator(self.acl)

    def _params(self):
        return {
            'path': self.path,
            'mimeType': self.mime_type,
            'acl': self.acl,
        }



import json

from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.callback import Callback
from media_platform.service.file_descriptor import ACL, FileDescriptor, FileMimeType
from media_platform.service.file_service.upload_file_response import _UploadFileResponse
from media_platform.service.file_service.upload_url_request import UploadUrlRequest
from media_platform.service.lifecycle import Lifecycle
from media_platform.service.media_platform_request import MediaPlatformRequest


class UploadFileRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(UploadFileRequest, self).__init__(authenticated_http_client, 'POST', base_url, FileDescriptor)

        self.path = None
        self.mime_type = FileMimeType.defualt
        self.acl = ACL.public
        self.lifecycle = None
        self.callback = None

        self.content = None

    def set_path(self, path):
        # type: (str) -> UploadFileRequest
        self.path = path
        return self

    def set_mime_type(self, mime_type):
        # type: (str) -> UploadFileRequest
        self.mime_type = mime_type
        return self

    def set_acl(self, acl):
        # type: (ACL) -> UploadFileRequest
        self.acl = acl
        return self

    def set_lifecycle(self, lifecycle):
        # type: (Lifecycle) -> UploadFileRequest
        self.lifecycle = lifecycle
        return self

    def set_callback(self, callback):
        # type: (Callback) -> UploadFileRequest
        self.callback = callback
        return self

    def set_content(self, content):
        self.content = content
        return self

    def validate(self):
        FileDescriptor.path_validator(self.path)
        FileDescriptor.acl_validator(self.acl)

    def execute(self):
        # type: () -> FileDescriptor
        self.validate()

        upload_url = UploadUrlRequest(self.authenticated_http_client, self.url).set_path(self.path).set_acl(
            self.acl
        ).set_mime_type(self.mime_type).execute()

        params = self._params()
        params.update({
            'uploadToken': upload_url.upload_token
        })

        response = self.authenticated_http_client.post_data(upload_url.upload_url, self.content, self.mime_type, params,
                                                            _UploadFileResponse)

        return response.file_descriptors[0]

    def _params(self):
        # type: () -> dict
        return {
            'path': self.path,
            'mimeType': self.mime_type,
            'acl': self.acl,
            'lifecycle': json.dumps(self.lifecycle.serialize()) if self.lifecycle else None,
            'callback': json.dumps(self.callback.serialize()) if self.callback else None,
        }



from media_platform.metadata.file_metadata import FileMetadata, MediaType
from media_platform.metadata.video.video_basic import VideoBasic
from media_platform.service.file_descriptor import FileDescriptor


class VideoFileMetadata(FileMetadata):
    def __init__(self, file_descriptor, basic=None):
        super(VideoFileMetadata, self).__init__(MediaType.video, file_descriptor, basic)

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> VideoFileMetadata
        if data['mediaType'] != MediaType.video:
            raise ValueError('not video metadata')

        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])
        basic = VideoBasic.deserialize(data.get('basic')) if data.get('basic') else None

        return VideoFileMetadata(file_descriptor, basic)



from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.file_service.copy_file_request import CopyFileRequest
from media_platform.service.file_service.create_file_request import CreateFileRequest
from media_platform.service.file_service.create_files_request import CreateFilesRequest
from media_platform.service.file_service.delete_file_request import DeleteFileRequest
from media_platform.service.file_service.download_file_request import DownloadFileRequest
from media_platform.service.file_service.extract_metadata_request import ExtractMetadataRequest
from media_platform.service.file_service.file_list_request import FileListRequest
from media_platform.service.file_service.file_metadata_request import FileMetadataRequest
from media_platform.service.file_service.file_request import FileRequest
from media_platform.service.file_service.import_file_request import ImportFileRequest
from media_platform.service.file_service.update_file_request import UpdateFileRequest
from media_platform.service.file_service.upload_configuration_request import UploadConfigurationRequest
from media_platform.service.file_service.upload_file_request import UploadFileRequest
from media_platform.service.file_service.upload_file_v2_request import UploadFileV2Request
from media_platform.service.file_service.upload_url_request import UploadUrlRequest
from media_platform.service.media_platform_service import MediaPlatformService


class FileService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client, app_id, app_authenticator):
        # type: (str, AuthenticatedHTTPClient, str, AppAuthenticator) -> None
        super(FileService, self).__init__(domain, authenticated_http_client)

        self._app_id = app_id
        self._app_authenticator = app_authenticator

    def file_request(self):
        # type: () -> FileRequest
        return FileRequest(self._authenticated_http_client, self._base_url)

    def create_file_request(self):
        # type: () -> CreateFileRequest
        return CreateFileRequest(self._authenticated_http_client, self._base_url)

    def update_file_request(self):
        # type: () -> UpdateFileRequest
        return UpdateFileRequest(self._authenticated_http_client, self._base_url)

    def create_files_request(self):
        # type: () -> CreateFilesRequest
        return CreateFilesRequest(self._authenticated_http_client, self._base_url)

    def upload_url_request(self):
        # type: () -> UploadUrlRequest
        return UploadUrlRequest(self._authenticated_http_client, self._base_url)

    def upload_file_request(self):
        # type: () -> UploadFileRequest
        return UploadFileRequest(self._authenticated_http_client, self._base_url)

    def upload_configuration_request(self):
        # type: () -> UploadConfigurationRequest
        return UploadConfigurationRequest(self._authenticated_http_client, self._base_url)

    def upload_file_v2_request(self):
        # type: () -> UploadFileV2Request
        return UploadFileV2Request(self._authenticated_http_client, self._base_url)

    def import_file_request(self):
        # type: () -> ImportFileRequest
        return ImportFileRequest(self._authenticated_http_client, self._base_url)

    def delete_file_request(self):
        # type: () -> DeleteFileRequest
        return DeleteFileRequest(self._authenticated_http_client, self._base_url)

    def copy_file_request(self):
        # type: () -> CopyFileRequest
        return CopyFileRequest(self._authenticated_http_client, self._base_url)

    def download_file_request(self):
        # type: () -> DownloadFileRequest
        return DownloadFileRequest(self._app_id, self._app_authenticator, self._base_url)

    def file_list_request(self):
        # type: () -> FileListRequest
        return FileListRequest(self._authenticated_http_client, self._base_url)

    def file_metadata_request(self):
        # type: () -> FileMetadataRequest
        return FileMetadataRequest(self._authenticated_http_client, self._base_url)

    def extract_metadata_request(self):
        # type: () -> ExtractMetadataRequest
        return ExtractMetadataRequest(self._authenticated_http_client, self._base_url)



from media_platform.job.specification import Specification
from media_platform.lang.serialization import Serializable
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.flow_control_service.component import Component, ComponentType
from media_platform.service.source import Source


class OperationStatus(object):
    waiting = 'waiting'
    aborted = 'aborted'
    working = 'working'
    success = 'success'
    error = 'error'


class Operation(Component):
    def __init__(self, component_type, successors, specification, status, delete_sources=False, sources=None,
                 results=None, jobs=None, extra_results=None, error_message=None, error_code=None, state_id=None,
                 component_key=None):
        # type: (ComponentType, [str], Specification, OperationStatus, bool, [Source], [FileDescriptor] or [dict], [str], dict, str, int, str, str) -> None
        super(Operation, self).__init__(component_type, successors, specification, delete_sources, sources=sources)
        self.status = status
        self.results = results or []
        self.jobs = jobs or []
        self.extra_results = extra_results or {}
        self.error_message = error_message
        self.error_code = error_code
        self.state_id = state_id
        self.component_key = component_key

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Operation
        o = Component.deserialize(data)
        o.__class__ = cls

        o.status = data['status']
        o.results = [FileDescriptor.deserialize(r) for r in data.get('results', [])]
        o.jobs = data.get('jobs', [])
        o.extra_results = data.get('extraResults', {})
        o.error_message = data.get('errorMessage')
        o.error_code = data.get('errorCode')
        o.state_id = data.get('stateId')
        o.component_key = data.get('componentKey')
        return o

    def serialize(self):
        data = super(Operation, self).serialize()

        data.update({
            'status': self.status,
            'results': [r.serialize() if isinstance(r, Serializable) else r for r in self.results],
            'jobs': self.jobs,
            'extraResults': self.extra_results
        })

        if self.error_code:
            data['errorCode'] = self.error_code

        if self.error_message:
            data['errorMessage'] = self.error_message

        if self.state_id:
            data['stateId'] = self.state_id

        if self.component_key:
            data['componentKey'] = self.component_key

        return data



from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.job.job import Job


class ImageOperationSpecification(Specification):
    def __init__(self, command, destination):
        # type: (str, Destination) -> None
        super(ImageOperationSpecification, self).__init__()

        self.command = command
        self.destination = destination

    def serialize(self):
        # type: () -> dict
        return {
            'command': self.command,
            'destination': self.destination.serialize()
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ImageOperationSpecification
        destination = Destination.deserialize(data['destination'])

        return ImageOperationSpecification(data['command'], destination)


class ImageOperationJob(Job):
    type = JobType.image_operation
    specification_type = ImageOperationSpecification



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.media_platform_service import MediaPlatformService
from media_platform.service.transcode_service.playlist_request import PlaylistRequest
from media_platform.service.transcode_service.transcode_request import TranscodeRequest


class TranscodeService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None
        super(TranscodeService, self).__init__(domain, authenticated_http_client)

    def transcode_request(self):
        # type: () -> TranscodeRequest
        return TranscodeRequest(self._authenticated_http_client, self._base_url)

    def playlist_request(self):
        # type: () -> PlaylistRequest
        return PlaylistRequest(self._domain)

    # todo: packager ?



from media_platform.service.source import Source
from media_platform.job.specification import Specification


class AddSourcesSpecification(Specification):
    def __init__(self, sources):
        # type: ([Source]) -> None
        self.sources = sources

    def serialize(self):
        return {
            'sources': [s.serialize() for s in self.sources]
        }

    @classmethod
    def deserialize(cls, data):
        return cls([Source.deserialize(s) for s in data['sources']])



from typing import Type

from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.lang.serialization import Deserializable


class MediaPlatformRequest(object):
    def __init__(self, authenticated_http_client, method, url, response_payload_type=None):
        # type: (AuthenticatedHTTPClient, str, str, Type[Deserializable] or None) -> None
        self.authenticated_http_client = authenticated_http_client

        self.method = method
        self.url = url
        self.response_payload_type = response_payload_type

    def execute(self):
        # type: () -> Deserializable or None

        self.validate()

        if self.method == 'GET':
            return self.authenticated_http_client.get(self.url, self._params(), self.response_payload_type)
        elif self.method == 'POST':
            return self.authenticated_http_client.post(self.url, self._params(), self.response_payload_type)
        elif self.method == 'PUT':
            return self.authenticated_http_client.put(self.url, self._params(), self.response_payload_type)
        elif self.method == 'DELETE':
            return self.authenticated_http_client.delete(self.url, self._params(), self.response_payload_type)
        else:
            raise ValueError('method not supported %s' % self.method)

    # override for request pre-flight check
    def validate(self):
        pass

    # noinspection PyMethodMayBeStatic
    def _params(self):
        # type: () -> dict
        return {}



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.job_service.job_group_request import JobGroupRequest
from media_platform.service.job_service.job_list_request import JobListRequest
from media_platform.service.job_service.job_request import JobRequest
from media_platform.service.media_platform_service import MediaPlatformService


class JobService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None
        super(JobService, self).__init__(domain, authenticated_http_client)

    def job_request(self):
        # type: () -> JobRequest
        return JobRequest(self._authenticated_http_client, self._base_url)

    def job_group_request(self):
        # type: () -> JobGroupRequest
        return JobGroupRequest(self._authenticated_http_client, self._base_url)

    def job_list_request(self):
        # type: () -> JobListRequest
        return JobListRequest(self._authenticated_http_client, self._base_url)



from jose import jwt

from media_platform.auth.token import Token


def sign_jwt(token, secret):
    # type: (Token, str) -> str

    return jwt.encode(token.to_claims(), secret, algorithm='HS256')


def decode_jwt(shared_secret, urn, signed_token):
    # type: (str, str, str) -> Token

    claims = jwt.decode(signed_token, shared_secret, subject=urn, options={
        'verify_aud': False
    })

    return Token.from_claims(claims)



from media_platform.job.convert_font_job import ConvertFontSpecification
from media_platform.job.create_archive_job import CreateArchiveSpecification
from media_platform.job.extract_archive.extract_archive_job import ExtractArchiveSpecification
from media_platform.job.extract_poster_job import ExtractPosterSpecification
from media_platform.job.extract_storyboard_job import ExtractStoryboardSpecification
from media_platform.job.import_file_job import ImportFileSpecification
from media_platform.job.replace_extra_metadata_job import ReplaceAudioExtraMetadataSpecification
from media_platform.job.specification import Specification
from media_platform.job.transcode_job import TranscodeSpecification
from media_platform.lang.serialization import Deserializable, Serializable
from media_platform.service.callback import Callback
from media_platform.service.flow_control_service.specifications.add_sources_specification import AddSourcesSpecification
from media_platform.service.flow_control_service.specifications.copy_file_specification import CopyFileSpecification
from media_platform.service.source import Source


class ComponentType(object):
    create_archive = 'archive.create'
    extract_archive = 'archive.extract'
    transcode = 'av.transcode'
    extract_poster = 'av.poster'
    extract_storyboard = 'av.storyboard'
    playlist = 'av.create_urlset'
    import_file = 'file.import'
    copy_file = 'file.copy'
    replace_extra_metadata = 'av.extra_metadata.replace'
    convert_font = 'font.convert'
    add_sources = 'flow.add_sources'
    group_wait = 'flow.group_wait'


_SPECIFICATIONS = {
    ComponentType.create_archive: CreateArchiveSpecification,
    ComponentType.extract_archive: ExtractArchiveSpecification,
    ComponentType.transcode: TranscodeSpecification,
    ComponentType.extract_poster: ExtractPosterSpecification,
    ComponentType.extract_storyboard: ExtractStoryboardSpecification,
    ComponentType.playlist: None,
    ComponentType.import_file: ImportFileSpecification,
    ComponentType.copy_file: CopyFileSpecification,
    ComponentType.replace_extra_metadata: ReplaceAudioExtraMetadataSpecification,
    ComponentType.convert_font: ConvertFontSpecification,
    ComponentType.add_sources: AddSourcesSpecification,
    ComponentType.group_wait: None,
}


class Component(Serializable, Deserializable):
    def __init__(self, component_type, successors=None, specification=None, delete_sources=False, callback=None, sources=None):
        # type: (ComponentType, [str], Specification, bool, Callback, [Source]) -> None
        super(Component, self).__init__()

        self.type = component_type
        self.successors = successors or []
        self.specification = specification
        self.delete_sources = delete_sources
        self.callback = callback
        self.sources = sources or []

    def serialize(self):
        # type: () -> dict
        return {
            'type': self.type,
            'successors': self.successors,
            'specification': self.specification.serialize() if self.specification else None,
            'deleteSources': self.delete_sources,
            'callback': self.callback.serialize() if self.callback else None,
            'sources': [s.serialize() for s in self.sources]
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Component
        specification_type = _SPECIFICATIONS[data['type']]
        specification = specification_type.deserialize(data['specification']) if specification_type else None

        callback_data = data.get('callback')
        callback = Callback.deserialize(callback_data) if callback_data else None
        sources = [Source.deserialize(s) for s in data.get('sources', [])]

        return cls(data['type'],
                   data.get('successors', []),
                   specification,
                   data.get('deleteSources', False),
                   callback,
                   sources)



from media_platform.lang.serialization import Deserializable
from media_platform.service.flow_control_service.component import ComponentType


class FlowError(Deserializable):
    def __init__(self, component_type, component_key, message=None):
        # type: (ComponentType, str, str) -> None
        super(FlowError, self).__init__()
        self.component_type = component_type
        self.component_key = component_key
        self.message = message

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> FlowError

        return FlowError(data['componentType'], data['componentKey'], data.get('message'))

    def serialize(self):
        return {
            'componentType': self.component_type,
            'componentKey': self.component_key,
            'message': self.message
        }


from media_platform.job.job import Job
# noinspection PyProtectedMember
from media_platform.job.job_deserializer import _JobDeserializer
from media_platform.lang.serialization import Deserializable


class JobList(Deserializable):
    def __init__(self, next_page_token, jobs):
        # type: (str, [Job]) -> None
        super(JobList, self).__init__()

        self.next_page_token = next_page_token
        self.jobs = jobs

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> JobList

        jobs = [_JobDeserializer.deserialize(j) for j in data['jobs']]

        return JobList(data['nextPageToken'], jobs)



from media_platform.lang.serialization import Deserializable
from media_platform.service.file_descriptor import FileDescriptor


class _UploadFileResponse(Deserializable):
    def __init__(self, file_descriptors):
        # type: ([FileDescriptor]) -> None
        super(_UploadFileResponse, self).__init__()

        self.file_descriptors = file_descriptors

    @classmethod
    def deserialize(cls, data):
        return _UploadFileResponse(
            [FileDescriptor.deserialize(item) for item in data]
        )



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.job import Job
from media_platform.service.job_service.job_group_response import _JobGroupResponse
from media_platform.service.media_platform_request import MediaPlatformRequest


class JobGroupRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(JobGroupRequest, self).__init__(authenticated_http_client, 'GET', base_url + '/jobs/', _JobGroupResponse)

        self.group_id = None

        self._url = base_url + '/jobs/groups/'

    def set_group_id(self, group_id):
        # type: (str) -> JobGroupRequest
        self.group_id = group_id
        return self

    def execute(self):
        # type: () -> [Job]

        self.url = self._url + self.group_id

        return super(JobGroupRequest, self).execute().jobs



from media_platform.lang.serialization import Serializable, Deserializable


class FontBasic(Serializable, Deserializable):
    def __init__(self, font_type=None, name=None, family=None):
        # type: (str, str, str) -> None
        super(FontBasic, self).__init__()
        self.font_type = font_type
        self.name = name
        self.family = family

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> FontBasic
        return FontBasic(data['fontType'], data.get('name'), data.get('family'))

    def serialize(self):
        # type: () -> dict
        return {
            'fontType': self.font_type,
            'name': self.name,
            'family': self.family
        }



from datetime import datetime

from media_platform.job.result.job_result import JobResult
from media_platform.job.result.job_result_deserializer import JobResultDeserializer
from media_platform.job.specification import Specification
from media_platform.lang import datetime_serialization
from media_platform.lang.serialization import Deserializable, Serializable
from media_platform.service.callback import Callback
from media_platform.service.source import Source


class JobStatus(object):
    pending = 'pending'
    working = 'working'
    success = 'success'
    error = 'error'


class JobID(Deserializable):
    def __init__(self, group_id, job_key):
        # type: (str, str) -> None
        self.group_id = group_id
        self.job_key = job_key

    @classmethod
    def deserialize(cls, data):
        # type: (str) -> JobID
        parts = data.split('_')
        return JobID(parts[0], parts[1])


class Job(Deserializable, Serializable):
    specification_type = None
    type = None

    def __init__(self, job_id, issuer, status, specification, sources=None, callback=None, flow_id=None,
                 result=None, date_created=None, date_updated=None):
        # type: (str, str, str, Specification or dict, [Source], Callback, str, JobResult, datetime, datetime) -> None

        _id = JobID.deserialize(job_id)

        self.id = job_id
        self.group_id = _id.group_id
        self.issuer = issuer
        self.status = status
        self.specification = specification
        self.sources = sources
        self.callback = callback
        self.flow_id = flow_id
        self.result = result
        self.date_created = date_created
        self.date_updated = date_updated

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Job

        sources = [Source.deserialize(source) for source in data['sources']]
        date_created = datetime_serialization.deserialize(data['dateCreated'])
        date_updated = datetime_serialization.deserialize(data['dateUpdated'])
        callback_data = data.get('callback')
        callback = Callback.deserialize(callback_data) if callback_data else None
        specification = data['specification']
        if cls.specification_type:
            specification = cls.specification_type.deserialize(specification)

        result_data = data.get('result')
        result = JobResultDeserializer.deserialize(data['type'], result_data) if result_data else None

        job = cls(data['id'], data['issuer'], data['status'], specification, sources, callback,
                  data.get('flowId'), result, date_created, date_updated)
        job.type = data['type']
        return job

    def serialize(self):
        return {
            'type': self.type,
            'groupId': self.group_id,
            'id': self.id,
            'issuer': self.issuer,
            'status': self.status,
            'specification': (self.specification.serialize() if isinstance(self.specification, Serializable)
                              else self.specification),
            'sources': [s.serialize() for s in self.sources],
            'callback': self.callback.serialize() if self.callback else None,
            'flowId': self.flow_id,
            'result': self.result.serialize() if self.result else None,
            'dateCreated': datetime_serialization.serialize(self.date_created),
            'dateUpdated': datetime_serialization.serialize(self.date_updated),
        }



from media_platform.auth.jwt_coder import sign_jwt, decode_jwt
from media_platform.auth.token import Token


class AppAuthenticator(object):
    def __init__(self, app_id, shared_secret):
        # type: (str, str) -> None

        self._shared_secret = shared_secret
        self._app_urn = 'urn:app:' + app_id

    def default_token(self):
        # type: () -> Token

        return Token(self._app_urn, self._app_urn)

    def default_signed_token(self):
        # type: () -> str
        token = self.default_token()

        return self.sign_token(token)

    def sign_token(self, token):
        # type: (Token) -> str

        return sign_jwt(token, self._shared_secret)

    def decode_token(self, signed_token):
        # type: (str) -> Token

        return decode_jwt(self._shared_secret, self._app_urn, signed_token)



from typing import Optional

from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.file_descriptor import FileDescriptor, ACL
from media_platform.service.lifecycle import Lifecycle


class Destination(Serializable, Deserializable):
    def __init__(self, path=None, directory=None, acl=ACL.public, lifecycle=None, bucket=None):
        # type: (Optional[str], Optional[str], Optional[ACL], Optional[Lifecycle], Optional[str]) -> None
        super(Destination, self).__init__()

        self._validate_values(path, directory, acl)

        self.path = path
        self.directory = directory
        self.acl = acl or ACL.public
        self.lifecycle = lifecycle
        self.bucket = bucket

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Destination

        lifecycle_data = data.get('lifecycle')
        lifecycle = Lifecycle.deserialize(lifecycle_data) if lifecycle_data else None

        return cls(data.get('path'), data.get('directory'), data['acl'], lifecycle, data.get('bucket'))

    def serialize(self):
        # type: () -> dict
        return {
            'path': self.path,
            'directory': self.directory,
            'acl': self.acl,
            'lifecycle': self.lifecycle.serialize() if self.lifecycle else None,
            'bucket': self.bucket
        }

    @staticmethod
    def _validate_values(path, directory, acl):
        # type: (str, str, ACL) -> None

        if not (path or directory):
            raise ValueError('path or directory must be specified')

        FileDescriptor.acl_validator(acl)

        if path:
            FileDescriptor.path_validator(path)

        if directory:
            FileDescriptor.path_validator(directory)



from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult


class CreateArchiveResult(JobResult):
    type = JobType.create_archive

    def __init__(self, code=None, message=None, created_file_descriptor=None):
        # type: (int, str, FileDescriptor) -> None
        super(CreateArchiveResult, self).__init__(code, message)
        self.created_file_descriptor = created_file_descriptor

    @classmethod
    def deserialize(cls, data):
        # type: (dict or None) -> CreateArchiveResult or None
        if not data:
            return None

        payload = data.get('payload')
        result = JobResult.deserialize(data)
        result.created_file_descriptor = FileDescriptor.deserialize(payload) if payload else None
        result.__class__ = CreateArchiveResult
        return result

    def serialize(self):
        # type: () -> dict
        data = super(CreateArchiveResult, self).serialize()
        data['payload'] = self.created_file_descriptor.serialize() if self.created_file_descriptor else None
        return data





from media_platform.service.file_descriptor import FileDescriptor
from media_platform.job.job_type import JobType
from media_platform.job.result.job_result import JobResult


class ExtractStoryboardResult(JobResult):
    type = JobType.extract_storyboard

    def __init__(self, code=None, message=None, file_descriptors=None):
        # type: (int, str, [FileDescriptor]) -> None
        super(ExtractStoryboardResult, self).__init__(code, message)

        self.payload = file_descriptors or []

    @classmethod
    def deserialize(cls, data):
        if data is None:
            return None

        result = JobResult.deserialize(data)
        payload_data = data.get('payload') or []
        result.payload = [FileDescriptor.deserialize(d) for d in payload_data]
        result.__class__ = ExtractStoryboardResult

        return result

    def serialize(self):
        data = super(ExtractStoryboardResult, self).serialize()
        data['payload'] = [f.serialize() for f in self.payload]

        return data



from media_platform.lang.serialization import Serializable, Deserializable


class Clipping(Serializable, Deserializable):
    def __init__(self, start=None, duration=None, fade_in_duration=None, fade_out_duration=None, fade_in_offset=None,
                 fade_out_offset=None):
        # type: (int, int, int, int, int, int) -> None
        self.start = start
        self.duration = duration
        self.fade_in_duration = fade_in_duration
        self.fade_out_duration = fade_out_duration
        self.fade_in_offset = fade_in_offset
        self.fade_out_offset = fade_out_offset

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Clipping
        return cls(data.get('start'), data.get('duration'), data.get('fadeInDuration'), data.get('fadeOutDuration'),
                   data.get('fadeInOffset'), data.get('fadeOutOffset'))

    def serialize(self):
        # type: () -> dict
        return {
            'start': self.start,
            'duration': self.duration,
            'fadeInDuration': self.fade_in_duration,
            'fadeOutDuration': self.fade_out_duration,
            'fadeInOffset': self.fade_in_offset,
            'fadeOutOffset': self.fade_out_offset
        }

    # todo: Add validate()



from media_platform.job.specification import Specification
from media_platform.lang.serialization import Serializable, Deserializable


class VideoSpecification(Specification):
    def __init__(self, codec, resolution=None, frame_rate=None, filters=None, frame_rate_fraction=None):
        # type: (VideoCodec, Resolution or None, float, [ImageFilter] or None, str or None) -> None
        self.codec = codec
        self.resolution = resolution
        self.frame_rate = frame_rate
        self.filters = filters
        self.frame_rate_fraction = frame_rate_fraction

    @classmethod
    def deserialize(cls, data):
        codec = VideoCodec.deserialize(data['codec'])

        resolution = Resolution.deserialize(data.get('resolution')) if data.get('resolution') else None

        filters_data = data.get('filters')
        filters = [ImageFilter.deserialize(f) for f in filters_data] if filters_data else None

        return VideoSpecification(codec, resolution, data.get('frameRate'), filters, data.get('frameRateFraction'))

    def serialize(self):
        data = {
            'codec': self.codec.serialize(),
            'resolution': self.resolution.serialize() if self.resolution else None,
            'filters': [f.serialize() for f in self.filters] if self.filters else None,
        }

        if self.frame_rate_fraction:
            data['frameRateFraction'] = self.frame_rate_fraction
        else:
            data['frameRate'] = self.frame_rate

        return data

    def validate(self):
        if self.resolution:
            self.resolution.validate()


class Resolution(Serializable, Deserializable):
    def __init__(self, width=None, height=None, scaling=None, sample_aspect_ratio=None):
        # type: (int, int, ImageScaling or None, str or None) -> None
        self.width = width
        self.height = height

        self.scaling = scaling
        self.sample_aspect_ratio = sample_aspect_ratio  # defaults to '1:1'

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Resolution
        scaling_data = data.get('scaling')
        scaling = ImageScaling.deserialize(scaling_data) if scaling_data else None

        return Resolution(data.get('width'), data.get('height'), scaling, data.get('sampleAspectRatio'))

    def serialize(self):
        # type: () -> dict
        return {
            'width': self.width,
            'height': self.height,
            'scaling': self.scaling.serialize() if self.scaling else None,
            'sampleAspectRatio': self.sample_aspect_ratio
        }

    def validate(self):
        if self.width and self.width % 2:
            raise ValueError('resolution width %s not divisible by 2' % self.width)

        if self.height and self.height % 2:
            raise ValueError('resolution height %s not divisible by 2' % self.height)


class VideoCodec(Serializable, Deserializable):
    def __init__(self, name, profile, level, crf, max_rate, gop=None, preset=None):
        # type: (str, str, str, int, float, GOP or None, str or None) -> None
        self.name = name
        self.profile = profile
        self.level = level
        self.crf = crf
        self.max_rate = max_rate
        self.gop = gop
        self.preset = preset

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> VideoCodec
        gop_data = data.get('gop')
        gop = GOP.deserialize(gop_data) if gop_data else None

        return VideoCodec(data['name'], data['profile'], data['level'], data['crf'], data['maxRate'], gop,
                          data.get('preset'))

    def serialize(self):
        # type: () -> dict
        return {
            'name': self.name,
            'profile': self.profile,
            'level': self.level,
            'crf': self.crf,
            'maxRate': self.max_rate,
            'gop': self.gop.serialize() if self.gop else None,
            'preset': self.preset
        }


class GOP(Serializable, Deserializable):
    def __init__(self, scene_cut, key_interval, min_key_interval, b_frames, b_pyramid, b_adapt, ref_frame):
        # type: (int or None, int or None, int or None, int or None, int or None, int or None, int or None) -> None
        super(GOP, self).__init__()

        self.scene_cut = scene_cut

        self.key_interval = key_interval
        self.min_key_interval = min_key_interval

        self.b_frames = b_frames
        self.b_pyramid = b_pyramid
        self.b_adapt = b_adapt

        self.ref_frame = ref_frame

    @classmethod
    def deserialize(cls, data):
        return cls(
            data['sceneCut'],
            data['keyInterval'],
            data['minKeyInterval'],
            data['bFrames'],
            data['bPyramid'],
            data['bAdapt'],
            data['refFrame']
        )

    def serialize(self):
        return {
            'sceneCut': self.scene_cut,
            'keyInterval': self.key_interval,
            'minKeyInterval': self.min_key_interval,
            'bFrames': self.b_frames,
            'bPyramid': self.b_pyramid,
            'bAdapt': self.b_adapt,
            'refFrame': self.ref_frame
        }


class ImageScaling(Serializable, Deserializable):
    def __init__(self, algorithm):
        super(ImageScaling, self).__init__()

        self.algorithm = algorithm

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ImageScaling
        return cls(data['algorithm'])

    def serialize(self):
        # type: () -> dict
        return {
            'algorithm': self.algorithm
        }


class ImageFilter(Serializable, Deserializable):
    def __init__(self, name, settings):
        # type: (str, dict) -> None
        super(ImageFilter, self).__init__()

        self.name = name  # 'unsharp'
        self.settings = settings  # '{"value": "5:5:0.5:3:3:0.0"}'

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ImageFilter
        return cls(data['name'], data['settings'])

    def serialize(self):
        # type: () -> dict
        return {
            'name': self.name,
            'settings': self.settings
        }



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.audio_service.extract_metadata_request import ExtractMetadataRequest
from media_platform.service.audio_service.replace_extra_metadata_request import ReplaceExtraMetadataSyncRequest, ReplaceExtraMetadataAsyncRequest

from media_platform.service.media_platform_service import MediaPlatformService


class AudioService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None
        super(AudioService, self).__init__(domain, authenticated_http_client)

    def extract_metadata_request(self):
        # type: () -> ExtractMetadataRequest
        return ExtractMetadataRequest(self._authenticated_http_client, self._base_url)

    def replace_extra_metadata_sync_request(self):
        return ReplaceExtraMetadataSyncRequest(self._authenticated_http_client, self._base_url)

    def replace_extra_metadata_async_request(self):
        return ReplaceExtraMetadataAsyncRequest(self._authenticated_http_client, self._base_url)



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.image_service.extract_features_request import ExtractFeaturesRequest
from media_platform.service.image_service.image_operation_request import ImageOperationRequest
from media_platform.service.media_platform_service import MediaPlatformService


class ImageService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client):
        # type: (str, AuthenticatedHTTPClient) -> None
        super(ImageService, self).__init__(domain, authenticated_http_client)

    def extract_features_request(self):
        # type: () -> ExtractFeaturesRequest
        return ExtractFeaturesRequest(self._authenticated_http_client, self._base_url)

    def image_operation_request(self):
        # type: () -> ImageOperationRequest
        return ImageOperationRequest(self._authenticated_http_client, self._base_url)



from media_platform.metadata.file_metadata import FileMetadata, MediaType
from media_platform.metadata.font.font_basic import FontBasic
from media_platform.service.file_descriptor import FileDescriptor


class FontFileMetadata(FileMetadata):
    def __init__(self, file_descriptor, basic=None):
        # type: (FileDescriptor, FontBasic) -> None
        super(FontFileMetadata, self).__init__(MediaType.font, file_descriptor, basic)

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> FontFileMetadata
        if data['mediaType'] != MediaType.font:
            raise ValueError('not font metadata')

        file_descriptor = FileDescriptor.deserialize(data['fileDescriptor'])
        basic = FontBasic.deserialize(data.get('basic')) if data.get('basic') else None

        return FontFileMetadata(file_descriptor, basic)



from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.job.job import Job


class ImportFileSpecification(Specification):
    def __init__(self, source_url, destination):
        # type: (str, Destination) -> None
        super(ImportFileSpecification, self).__init__()

        self.source_url = source_url
        self.destination = destination

    def serialize(self):
        # type: () -> dict
        return {
            'sourceUrl': self.source_url,
            'destination': self.destination.serialize()
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ImportFileSpecification
        destination = Destination.deserialize(data['destination'])

        return ImportFileSpecification(data['sourceUrl'], destination)


class ImportFileJob(Job):
    type = JobType.import_file
    specification_type = ImportFileSpecification




class JobType(object):
    transcode = 'urn:job:av.transcode'
    extract_poster = 'urn:job:av.poster'
    extract_storyboard = 'urn:job:av.storyboard'
    create_archive = 'urn:job:archive.create'
    extract_archive = 'urn:job:archive.extract'
    import_file = 'urn:job:import.file'
    replace_extra_metadata = 'urn:job:av.extra_metadata.replace'
    convert_font = 'urn:job:text.font.convert'
    subset_font = 'urn:job:text.font.subset'
    image_operation = 'urn:job:image.operation'



from media_platform.lang.serialization import Serializable, Deserializable
from media_platform.service.callback import Callback
from media_platform.service.source import Source


class ErrorStrategy(object):
    stop_on_error = 'stopOnError'
    continue_on_error = 'continueOnError'


class Invocation(Serializable, Deserializable):
    def __init__(self, entry_points, sources=None, callback=None, error_strategy=None):
        # type: ([str], [Source], Callback or None, ErrorStrategy) -> None
        super(Invocation, self).__init__()

        self.entry_points = entry_points
        self.sources = sources or []
        self.callback = callback
        self.error_strategy = error_strategy or ErrorStrategy.stop_on_error

    def serialize(self):
        # type: () -> dict
        return {
            'entryPoints': self.entry_points,
            'sources': [source.serialize() for source in self.sources],
            'callback': self.callback.serialize() if self.callback else None,
            'errorStrategy': self.error_strategy
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> Invocation
        sources = [Source.deserialize(s) for s in data['sources']]
        callback_data = data.get('callback')
        callback = Callback.deserialize(callback_data) if callback_data else None

        return cls(data['entryPoints'], sources, callback, data.get('errorStrategy'))



from media_platform.job.job_group import JobGroup


class SubsetFontJobGroup(JobGroup):

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> SubsetFontJobGroup

        return super(SubsetFontJobGroup, cls).deserialize(data)



from media_platform.auth.app_authenticator import AppAuthenticator
from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.service.archive_service.archive_manifest_url_request import ArchiveManifestUrlRequest
from media_platform.service.archive_service.create_archive_manifest_request import CreateArchiveManifestRequest
from media_platform.service.archive_service.create_archive_request import CreateArchiveRequest
from media_platform.service.archive_service.extract_archive_request import ExtractArchiveRequest
from media_platform.service.media_platform_service import MediaPlatformService


class ArchiveService(MediaPlatformService):
    def __init__(self, domain, authenticated_http_client, app_id, app_authenticator):
        # type: (str, AuthenticatedHTTPClient, str, AppAuthenticator) -> None
        super(ArchiveService, self).__init__(domain, authenticated_http_client)

        self._app_id = app_id
        self._app_authenticator = app_authenticator

    def create_archive_request(self):
        # type: () -> CreateArchiveRequest
        return CreateArchiveRequest(self._authenticated_http_client, self._base_url)

    def create_archive_manifest_request(self):
        # type: () -> CreateArchiveManifestRequest
        return CreateArchiveManifestRequest(self._authenticated_http_client, self._base_url)

    def archive_manifest_url_request(self):
        # type: () -> ArchiveManifestUrlRequest
        return ArchiveManifestUrlRequest(self._app_id, self._app_authenticator, self._domain)

    def extract_archive_request(self):
        # type: () -> ExtractArchiveRequest
        return ExtractArchiveRequest(self._authenticated_http_client, self._base_url)



from media_platform.lang.serialization import Deserializable
from media_platform.metadata.features.explicit_content import ExplicitContent
from media_platform.metadata.features.image_color import Color
from media_platform.metadata.features.label import Label
from media_platform.metadata.features.rectangle import Rectangle


class ImageFeatures(Deserializable):
    def __init__(self, labels=None, faces=None, colors=None, explicit_content=None):
        # type: ([Label], [Rectangle], [Color], [ExplicitContent]) -> None
        self.labels = labels or []
        self.faces = faces or []
        self.colors = colors or []
        self.explicit_content = explicit_content or []

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ImageFeatures
        labels = [Label.deserialize(label) for label in data.get('labels', [])]
        faces = [Rectangle.deserialize(face) for face in data.get('faces', [])]
        colors = [Color.deserialize(color) for color in data.get('colors', [])]
        explicit_content = [ExplicitContent.deserialize(content) for content in data.get('explicitContent', [])]

        return ImageFeatures(labels, faces, colors, explicit_content)



from media_platform.lang.serialization import Serializable, Deserializable


class Specification(Serializable, Deserializable):

    def serialize(self):
        raise NotImplementedError()

    @classmethod
    def deserialize(cls, data):
        raise NotImplementedError()

    # override for request pre-flight check
    def validate(self):
        pass



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.create_archive_job import ArchiveSource, CreateArchiveJob, ArchiveType
from media_platform.service.callback import Callback
from media_platform.service.destination import Destination
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class CreateArchiveRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(CreateArchiveRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/archive/create',
                                                   CreateArchiveJob)

        self.sources = []
        self.destination = None
        self.archive_type = 'zip'
        self.callback = None

    def set_sources(self, sources):
        # type: ([Source or ArchiveSource]) -> CreateArchiveRequest
        self.sources = sources
        return self

    def add_sources(self, *sources):
        # type: ([Source or ArchiveSource]) -> CreateArchiveRequest
        self.sources.extend(sources)
        return self

    def set_destination(self, destination):
        # type: (Destination) -> CreateArchiveRequest
        self.destination = destination
        return self

    def set_archive_type(self, archive_type):
        # type: (ArchiveType) -> CreateArchiveRequest
        self.archive_type = archive_type
        return self

    def set_callback(self, callback):
        # type: (Callback) -> CreateArchiveRequest
        self.callback = callback
        return self

    def execute(self):
        # type: () -> CreateArchiveJob
        return super(CreateArchiveRequest, self).execute()

    def _params(self):
        # type: () -> dict
        return {
            'sources': [source.serialize() for source in self.sources if source],
            'destination': self.destination.serialize(),
            'archiveType': self.archive_type,
            'jobCallback': self.callback.serialize() if self.callback else None
        }



from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.job.job import Job


class FontType(object):
    woff = 'woff'
    woff2 = 'woff2'
    ttf = 'ttf'
    otf = 'otf'

    values = [
        woff,
        woff2,
        ttf,
        otf
    ]

    @classmethod
    def has_value(cls, value):
        # type: (FontType or str) -> bool
        return value in cls.values


class FontSet(object):
    web = 'web'

    values = [
        web
    ]

    @classmethod
    def has_value(cls, value):
        # type: (FontSet or str) -> bool
        return value in cls.values


class ConvertFontSpecification(Specification):
    def __init__(self, destination, font_type=None, font_set=None):
        # type: (Destination, FontType, FontSet) -> None
        super(ConvertFontSpecification, self).__init__()
        self.destination = destination

        self.font_type = font_type
        self.font_set = font_set

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ConvertFontSpecification
        destination = Destination.deserialize(data['destination'])

        return ConvertFontSpecification(destination, data.get('fontType'), data.get('fontSet'))

    def serialize(self):
        # type: () -> dict
        return {
            'destination': self.destination.serialize(),
            'fontType': self.font_type,
            'fontSet': self.font_set
        }

    def validate(self):
        if not self.font_type and not self.font_set:
            raise ValueError('must select type or set')

        if self.font_set and not self.destination.directory:
            raise ValueError('must provide destination directory for font set')

        if self.font_type:
            self._validate_font_type(self.font_type)

        if self.font_set:
            self._validate_font_set(self.font_set)

    @staticmethod
    def _validate_font_type(font_type):
        # type: (FontType or str) -> None
        if not FontType.has_value(font_type):
            raise ValueError('font format must be one of: %s' % ', '.join(FontType.values))

    @staticmethod
    def _validate_font_set(font_set):
        # type: (FontSet or str) -> None
        if not FontSet.has_value(font_set):
            raise ValueError('font ste must be one of: %s' % ', '.join(FontSet.values))


class ConvertFontJob(Job):
    type = JobType.convert_font
    specification_type = ConvertFontSpecification



from typing import Optional

from media_platform.job.job_type import JobType
from media_platform.job.specification import Specification
from media_platform.service.destination import Destination
from media_platform.job.job import Job


class PosterImageFormat(object):
    jpeg = 'jpg'
    png = 'png'
    values = [jpeg, png]
    invalid_value_message = 'Image format must be one of: %s' % ', '.join(values)


class PosterFilter(object):
    transparent_crop = 'transparentCrop'
    values = [transparent_crop]
    invalid_value_message = 'Filters must be one of: %s' % ', '.join(values)


class ExtractPosterSpecification(Specification):
    def __init__(self, second, destination, image_format=PosterImageFormat.jpeg, percentage=None, filters=None):
        # type: (Optional[float], Destination, Optional[PosterImageFormat], Optional[float], Optional[PosterFilter]) -> None
        super(ExtractPosterSpecification, self).__init__()

        self.second = second
        self.destination = destination
        self.image_format = image_format or PosterImageFormat.jpeg
        self.percentage = percentage
        self.filters = filters or []

    def serialize(self):
        # type: () -> dict
        return {
            'second': self.second,
            'percentage': self.percentage,
            'destination': self.destination.serialize(),
            'format': self.image_format,
            'filters': self.filters
        }

    @classmethod
    def deserialize(cls, data):
        # type: (dict) -> ExtractPosterSpecification
        destination = Destination.deserialize(data['destination'])

        return ExtractPosterSpecification(data['second'], destination, data['format'], data.get('percentage'),
                                          data.get('filters'))

    def validate(self):
        self._validate_image_format()
        self._validate_time()
        self._validate_filters()

    def _validate_image_format(self):
        if self.image_format not in PosterImageFormat.values:
            raise ValueError(PosterImageFormat.invalid_value_message)

    def _validate_time(self):
        if self.second is None and self.percentage is None:
            raise ValueError('Must provide either second or percentage')

        if self.percentage is not None and (self.percentage < 0 or self.percentage > 100):
            raise ValueError('Percentage must be 0-100')

    def _validate_filters(self):
        invalid_filters = [f for f in self.filters if f not in PosterFilter.values]
        if invalid_filters:
            raise ValueError(PosterFilter.invalid_value_message)


class ExtractPosterJob(Job):
    type = JobType.extract_poster
    specification_type = ExtractPosterSpecification



from media_platform.http.authenticated_http_client import AuthenticatedHTTPClient
from media_platform.job.transcode_job import TranscodeSpecification
from media_platform.job.transcode_job_group import TranscodeJobGroup
from media_platform.service.callback import Callback
from media_platform.service.media_platform_request import MediaPlatformRequest
from media_platform.service.source import Source


class TranscodeRequest(MediaPlatformRequest):
    def __init__(self, authenticated_http_client, base_url):
        # type: (AuthenticatedHTTPClient, str) -> None
        super(TranscodeRequest, self).__init__(authenticated_http_client, 'POST', base_url + '/av/transcode',
                                               TranscodeJobGroup)

        self.sources = []
        self.specifications = []
        self.callback = None

    def set_sources(self, sources):
        # type: ([Source]) -> TranscodeRequest
        self.sources = sources
        return self

    def add_sources(self, *sources):
        # type: ([Source]) -> TranscodeRequest
        self.sources.extend(sources)
        return self

    def set_specifications(self, specifications):
        # type: ([TranscodeSpecification]) -> TranscodeRequest
        self.specifications = specifications
        return self

    def add_specifications(self, *specifications):
        # type: ([TranscodeSpecification]) -> TranscodeRequest
        self.specifications.extend(specifications)
        return self

    def set_callback(self, callback):
        # type: (Callback) -> TranscodeRequest
        self.callback = callback
        return self

    def validate(self):
        [specification.validate() for specification in self.specifications]

    def execute(self):
        # type: () -> TranscodeJobGroup
        return super(TranscodeRequest, self).execute()

    def _params(self):
        return {
            'sources': [source.serialize() for source in self.sources],
            'specifications': [specification.serialize() for specification in self.specifications],
            'jobCallback': self.callback.serialize() if self.callback else None
        }



from media_platform.media_platform_client import MediaPlatformClient
from media_platform.service.destination import Destination
from media_platform.service.file_descriptor import FileDescriptor
from media_platform.service.source import Source

Client = MediaPlatformClient

source = Source
destination = Destination

file_descriptor = FileDescriptor
